@@ -109,6 +109,36 @@ MODULES = OrderedDict((
         classname = 'SoL.module.Competitors')),
     ("ratedplayers", dict(
         classname = 'SoL.module.RatedPlayers')),
+    ("myclubs", dict(
+        classname = 'SoL.module.MyClubs',
+        shortcut = dict(
+            name = _('My clubs'),
+            iconCls = 'clubs-shortcut-icon',
+            moduleId = 'my-clubs-win'))),
+    ("mychampionships", dict(
+        classname = 'SoL.module.MyChampionships',
+        shortcut = dict(
+            name = _('My championships'),
+            iconCls = 'championships-shortcut-icon',
+            moduleId = 'my-championships-win'))),
+    ("mytourneys", dict(
+        classname = 'SoL.module.MyTourneys',
+        shortcut = dict(
+            name = _('My tourneys'),
+            iconCls = 'tourneys-shortcut-icon',
+            moduleId = 'my-tourneys-win'))),
+    ("myplayers", dict(
+        classname = 'SoL.module.MyPlayers',
+        shortcut = dict(
+            name = _('My players'),
+            iconCls = 'players-shortcut-icon',
+            moduleId = 'my-players-win'))),
+    ("myratings", dict(
+        classname = 'SoL.module.MyRatings',
+        shortcut = dict(
+            name = _('My ratings'),
+            iconCls = 'ratings-shortcut-icon',
+            moduleId = 'my-ratings-win'))),
 ))
 
 
