@@ -126,7 +126,7 @@ def competitors(request, results):
 
 @view_config(route_name="matches", renderer="json")
 @expose(Match,
-        fields=('board,description,score1,score2,turn,'
+        fields=('board,description,score1,score2,turn,final,'
                 'idmatch,idcompetitor1,idcompetitor2').split(','),
         metadata=dict(
     description=dict(label=_('Match'),
@@ -138,15 +138,14 @@ def competitors(request, results):
     board=dict(readonly=True, width=70),
     score1=dict(width=70),
     score2=dict(width=70),
+    final=dict(hidden=True, readonly=True),
     ))
 def matches(request, results):
     return results
 
 
 @view_config(route_name="ranking", renderer="json", request_param="turn")
-def ranking_for_turn(request):
-    from operator import itemgetter
-
+def rankingForTurn(request):
     try:
         sess = DBSession()
         params = request.params
@@ -155,16 +154,11 @@ def ranking_for_turn(request):
 
         tourney = sess.query(Tourney).get(idtourney)
 
-        ranking = [dict(idcompetitor=c.idcompetitor, description=c.description,
-                        points=r[0], netscore=r[1], totscore=r[2], bucholz=r[3],
-                        rate=c.rate, prize=0, player1Nationality=c.player1Nationality)
-                   for c, r in tourney.computeRanking(turn).items()
-                   if c is not None]
-        ranking.sort(key=itemgetter('points', 'bucholz', 'netscore', 'totscore', 'rate'),
-                     reverse=True)
-
-        for i in range(len(ranking)):
-            ranking[i]['rank'] = i+1
+        ranking = [dict(rank=i, idcompetitor=c.idcompetitor, description=c.description,
+                        points=r.points, netscore=r.netscore, totscore=r.totscore,
+                        bucholz=r.bucholz, rate=r.rate, prize=0,
+                        player1Nationality=c.player1Nationality)
+                   for i, (c, r) in enumerate(tourney.computeRanking(turn), 1)]
 
         return dict(success=True, message='Ok', count=len(ranking), root=ranking)
     except Exception as e: # pragma: no cover
@@ -195,11 +189,13 @@ def ranking_for_turn(request):
               hidden=True,
               align='right',
               type='int',
-              readonly=True),
+              readonly=True,
+              width=50),
     player1Nationality=dict(label=_('Nationality'),
                             hint=_('Nationality of the competitor.'),
                             hidden=True,
-                            readonly=True)
+                            readonly=True,
+                            width=40)
     ))
 def ranking(request, results):
     from operator import itemgetter
@@ -215,10 +211,14 @@ def ranking(request, results):
             'align': 'right',
             'type': 'int' })
     else:
-        results['root'].sort(key=itemgetter('prize', 'points', 'bucholz',
-                                            'netscore', 'totscore', 'rate'),
-                             reverse=True)
-        for i, r in enumerate(results['root'], 1):
+        ranking = results['root']
+        # Match the local ordering applied by the Ranking panel, so the "rank" field is
+        # reasonable: by ascending player's name, then by descending ranking position.
+        ranking.sort(key=itemgetter('description'))
+        ranking.sort(key=itemgetter('prize', 'points', 'bucholz',
+                                    'netscore', 'totscore', 'rate'),
+                     reverse=True)
+        for i, r in enumerate(ranking, 1):
             r['rank'] = i
     return results
 
@@ -250,11 +250,19 @@ def deleteFromTurn(request):
         fromturn = int(params['fromturn'])
 
         tourney = sess.query(Tourney).get(idtourney)
+
+        if tourney.prized:
+            tourney.resetPrizes()
+
         delmatches = [m for m in tourney.matches if m.turn >= fromturn]
         for match in delmatches:
             sess.delete(match)
 
         tourney.currentturn = fromturn-1
+        if tourney.finalturns:
+            tourney.finalturns = any(m for m in tourney.matches
+                                     if m.turn < fromturn and m.final)
+        tourney.countdownstarted = None
         sess.flush()
 
         # recompute the ranking
@@ -274,6 +282,7 @@ def deleteFromTurn(request):
     return dict(success=success, message=message,
                 currentturn=tourney.currentturn,
                 rankedturn=tourney.rankedturn,
+                finalturns=tourney.finalturns,
                 prized=tourney.prized)
 
 
@@ -307,6 +316,40 @@ def newTurn(request):
     return dict(success=success, message=message,
                 currentturn=tourney.currentturn,
                 rankedturn=tourney.rankedturn,
+                finalturns=tourney.finalturns,
+                prized=tourney.prized)
+
+@view_config(route_name="final_turn", renderer="json")
+@unauthorized_for_guest
+def finalTurn(request):
+    "Create final turn."
+
+    try:
+        sess = DBSession()
+        params = request.params
+        idtourney = int(params['idtourney'])
+
+        tourney = sess.query(Tourney).get(idtourney)
+        tourney.makeFinalTurn()
+
+        sess.flush()
+
+        success = True
+        message = 'Ok'
+    except OperationAborted as e: # pragma: no cover
+        message = str(e)
+        get_request_logger(request, logger).error("Couldn't create final turn: %s", message)
+        success = False
+    except Exception as e: # pragma: no cover
+        message = str(e)
+        get_request_logger(request, logger).critical("Couldn't create final turn: %s",
+                                                     message, exc_info=True)
+        success = False
+
+    return dict(success=success, message=message,
+                currentturn=tourney.currentturn,
+                rankedturn=tourney.rankedturn,
+                finalturns=tourney.finalturns,
                 prized=tourney.prized)
 
 
@@ -335,6 +378,7 @@ def updateRanking(request):
     return dict(success=success, message=message,
                 currentturn=tourney.currentturn,
                 rankedturn=tourney.rankedturn,
+                finalturns=tourney.finalturns,
                 prized=tourney.prized)
 
 
@@ -366,6 +410,7 @@ def assignPrizes(request):
     return dict(success=success, message=message,
                 currentturn=tourney.currentturn,
                 rankedturn=tourney.rankedturn,
+                finalturns=tourney.finalturns,
                 prized=tourney.prized)
 
 
@@ -394,6 +439,7 @@ def resetPrizes(request):
     return dict(success=success, message=message,
                 currentturn=tourney.currentturn,
                 rankedturn=tourney.rankedturn,
+                finalturns=tourney.finalturns,
                 prized=tourney.prized)
 
 
@@ -434,6 +480,25 @@ def replayToday(request):
     return dict(success=success, message=message)
 
 
+@view_config(route_name="clock", request_method="POST", renderer="json")
+def clockCountdownStarted(request):
+    "Register the countdown start timestamp."
+
+    sess = DBSession()
+    params = request.params
+    idtourney = int(params['idtourney'])
+    start = int(params['start'])
+    tourney = sess.query(Tourney).get(idtourney)
+    if not tourney.prized:
+        tourney.countdownstarted = start
+        logger.debug("Countdown for %s started", repr(tourney))
+        return dict(success=True, message="Ok")
+    else:
+        get_request_logger(request, logger).error("Attempt to start countdown for %s ignored",
+                                                  repr(tourney))
+        return dict(success=False, message="Tourney is prized")
+
+
 @view_config(route_name="clock", renderer="clock.mako")
 def clock(request):
     "Show the clock."
@@ -460,7 +525,9 @@ def clock(request):
                           ('/static/clock/clock.css',))],
             duration=tourney.duration,
             prealarm=tourney.prealarm,
-            currentturn=ordinal(tourney.currentturn))
+            currentturn=ordinal(tourney.currentturn),
+            starttime=tourney.countdownstarted or '',
+            notifystart=request.path_qs)
     except Exception as e: # pragma: no cover
         get_request_logger(request, logger).critical("Couldn't show the clock: %s", e,
                                                      exc_info=True)
